@@ -7,8 +7,6 @@ use 5.014;
 
 use DBI qw();
 
-use Term::Choose::Util qw();
-
 use App::DBBrowser::Credentials;
 use App::DBBrowser::Opt::DBGet;
 
@@ -30,7 +28,7 @@ sub get_db_driver {
 
 sub env_variables {
     my ( $sf ) = @_;
-    return [ qw( DBI_DSN DBI_USER DBI_PASS ) ];
+    return [ qw( DBI_USER DBI_PASS ) ];
 }
 
 
@@ -71,15 +69,12 @@ sub get_db_handle {
 
     my $cred = App::DBBrowser::Credentials->new( $sf->{i}, $sf->{o} );
     my $settings = { login_data => $login_data, env_var_yes => $env_var_yes };
-    my $dsn;
-    my $show_sofar = 'DB '. $db;
     # DBD::DB2 - Data Source Names:
     # Cataloged database connections can be done by passing the database alias, username, and password as parameters.
     # This method does not allow entering the host name, port number, etc but will require you to catalog the database
     # (local or remote) through DB2.
-    if ( ! $env_var_yes->{DBI_DSN} || ! exists $ENV{DBI_DSN} ) {
-        $dsn = "dbi:$sf->{i}{driver}:$db";
-    }
+    my $dsn = "dbi:$sf->{i}{driver}:$db";
+    my $show_sofar = 'DB '. $db;
     my $user = $cred->get_login( 'user', $show_sofar, $settings );
     $show_sofar .= "\n" . 'User: ' . $user if defined $user;
     my $passwd = $cred->get_login( 'pass', $show_sofar, $settings );
